/*

keypress.cpp
============

Keypress module for simulation of key presses originally developed by group 451 spring 2006 at
Aalborg University from the Pys60 sysinfo template. It was used to simulate hang up on Nokia 6600
in silent mode.

Copyright 2006 group 451 spring 2006 at Aalborg University
Copyright 2006 Cyke64 (cyke64@gmail.com)
Copyright 2008 Arkadiusz Wahlig (arkadiusz.wahlig@gmail.com)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
    
http://www.apache.org/licenses/LICENSE-2.0
    
Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License. 

Version history
===============
1.0.6 (16-07-2008) Arkadiusz Wahlig
	added send_raw_event function to send a key event behind the window server

1.0.5 (14-07-2008) Arkadiusz Wahlig
	rewritten using uikludges module sources (more complete, with build batch files)
	added ability to simulate single key up/down events (simulate_event)
	cleaned up this comment
	fixed indentation in key_modifiers.py (but I think we don't need this one
	anymore as the modifiers are defined in key_codes.py distributed with
	latest PyS60 (1.4.4)
	
1.0.2 (05-10-2006) Cyke64
	add modifiers
	add key_modifiers.py

1.0.1 (05-02-2006) Cyke64
	simulate any key press from key_codes.py
	
1.0.0 group 451 spring 2006 at Aalborg University
	simulate key press on right menu key

*/

#include <e32std.h>
#include <e32base.h>
#include <EIKBTGPC.H>
#include <avkon.hrh>
#include <aknnavi.h>
#include <aknnavide.h>
#include <eikspane.h>
#include <aknappui.h> 

#include "Python.h"
#include "symbian_python_ext_util.h"

static PyObject* keypress_simulate_key(PyObject* /*self*/, PyObject *args)
{
	TInt error;
	TUint keycode, scancode, modifiers=0;
	RWsSession ws;
	TKeyEvent key;

	if (!PyArg_ParseTuple(args, "ii|i", &keycode, &scancode, &modifiers))
		return NULL;
  
   	key.iCode = keycode;
	key.iScanCode = scancode;
	key.iModifiers = modifiers;
   
	Py_BEGIN_ALLOW_THREADS;
	TRAP(error,
		ws.Connect();
		ws.SimulateKeyEvent(key);
		ws.Close();
	);
	Py_END_ALLOW_THREADS;
    
	if (error)
		return SPyErr_SetFromSymbianOSErr(error);
   
	return Py_BuildValue("i", 1);
}

static PyObject* keypress_simulate_event(PyObject* /*self*/, PyObject *args)
{
	TInt error;
	TWsEvent event;
	TKeyEvent key;
	TInt type;
	TUint keycode, scancode, modifiers=0, repeats=0;
	
	if (!PyArg_ParseTuple(args, "iii|ii", &type, &keycode, &scancode, &modifiers, &repeats))
		return NULL;

   	key.iCode = keycode;
	key.iScanCode = scancode;
	key.iModifiers = modifiers;
	key.iRepeats = repeats;

	TRAP(error,
		event.SetType(type);
		*(event.Key()) = key;
		event.SetTimeNow();
		CCoeEnv::Static()->WsSession().SendEventToWindowGroup(
			CCoeEnv::Static()->WsSession().GetFocusWindowGroup(), event);
  	);
  	
  	if (error)
  		return SPyErr_SetFromSymbianOSErr(error);

	return Py_BuildValue("i", 1);
}

static PyObject* keypress_send_raw_event(PyObject* /*self*/, PyObject *args)
{
	TInt error;
	TInt type;
	TUint p1=0xffffffff, p2=0xffffffff;
	TRawEvent event;

	if (!PyArg_ParseTuple(args, "i|ii", &type, &p1, &p2))
		return NULL;
   
	Py_BEGIN_ALLOW_THREADS;
	TRAP(error,
		if (p1 != 0xffffffff) {
			if (p2 != 0xffffffff) {
				event.Set(TRawEvent::TType(type), p1, p2);
			}
			else {
				event.Set(TRawEvent::TType(type), p1);
			}
		}
		else {
			event.Set(TRawEvent::TType(type));
		}
		UserSvr::AddEvent(event);
	);
	Py_END_ALLOW_THREADS;
    
	if (error)
		return SPyErr_SetFromSymbianOSErr(error);
   
	return Py_BuildValue("i", 1);
}

static const PyMethodDef keypress_methods[] =
{
	{"simulate_key", (PyCFunction)keypress_simulate_key, METH_VARARGS},
	{"simulate_event", (PyCFunction)keypress_simulate_event, METH_VARARGS},
	{"send_raw_event", (PyCFunction)keypress_send_raw_event, METH_VARARGS},
	{0, 0} /* sentinel */
};

DL_EXPORT(void) init_keypress()
{
	//PyObject* module = 
	Py_InitModule("_keypress", (PyMethodDef*)keypress_methods);
}

#ifndef EKA2
GLDEF_C TInt E32Dll(TDllReason)
{
	return KErrNone;
}
#endif
